import {FC, ReactNode, useCallback, useEffect, useState} from "react";
import {Video, View} from "@tarojs/components";
import {brandApi, BrandRecord} from "@/api";
import Taro, {useReachBottom} from "@tarojs/taro";
import Empty from "@/components/empty/empty";
import Spinner from "@/components/spinner";
import Img from "@/components/image/image";

const BrandItem: FC<{ data: BrandRecord; onClick: VoidFunction }> = ({data, onClick}) => {
  let media: ReactNode
  if (data.introductory_video_resource) {
    media = <Video
      style={{width: '100%'}}
      src={data.introductory_video_resource.url}
    />
  } else if (data.brand_album) {
    media = <Img
      width={712}
      height={320}
      src={data.brand_album.split(",")[0]}
      mode="aspectFill"
      style={{background: '#ededed'}}
      onClick={onClick}
    />
  }

  return (
    <View className="bg-white flex flex-column justify-stretch mb-2_4 rounded-10 clip">
      {media}
      <View className="p-3" onClick={onClick}>
        <View className='font-weight mb-2 font-32 flex align-center'>
          <Img
            width={32}
            height={32}
            src={data.logo}
            mode='aspectFill'
            className="rounded-10 clip"
            style={{background: '#ededed'}}
          />
          <View className="ml-1 flex-1">
            <View className="text-row1 font-28">{data.name}</View>
          </View>
        </View>
        <View className="font-24 text-muted mb-4 text-row3" style={{lineHeight:1.4}}>
          {data.graphic_introduction}
        </View>
        <View className="flex gap20rpx font-24 text-muted">
          <View>2023/03.29 12:22:01</View>
          <View className="flex-1"></View>
          <View>文章：123</View>
          <View>点赞：234</View>
        </View>
      </View>
    </View>
  )
}

const BrandList: FC = () => {
  const [page, setPage] = useState(1)
  const [brands, setBrands] = useState<BrandRecord[]>([])
  const [total, setTotal] = useState(0)
  const [text, setText] = useState('')
  const [loading, setLoading] = useState(true)

  useEffect(() => {
    getData()
  }, [page])

  const getData = useCallback(async () => {
    try {
      const res = await brandApi.list(page, 10)
      if (page === 1) {
        if (res.list.length < 10) {
          setText('没有更多了~')
        } else {
          setText('上拉加载更多~')
        }
      }
      setTotal(res.total)
      setBrands([
        ...brands,
        ...res.list
      ])
    } catch (e) {
    }
    setLoading(false)
  }, [page])


  function jumpInfo(id: number) {
    Taro.navigateTo({url: `/pages/preview/brand/info/info?id=${id}`})
  }

  useReachBottom(useCallback(() => {
    if (brands?.length < total) {
      setPage(page + 1)
    } else {
      setText('没有更多了~')
    }
  }, [total, brands]))

  let content: ReactNode
  if (brands.length) {
    content = (
      <>
        {brands.map(d => <BrandItem data={d} key={d.id} onClick={() => jumpInfo(d.id)}/>)}
        <View style={{width: '710rpx', textAlign: 'center', color: '#999'}} className="font-28 mt-3">{text}</View>
      </>
    )
  } else {
    content = <Empty name='空空如也'/>
  }

  return (
    <View className='p-2'>
      <Spinner enable={loading} overlay/>
      {content}
    </View>
  )
}

export default BrandList
